@@ -46,6 +46,8 @@
 
 mod err;
 pub mod rsa;
+#[cfg(feature = "x509")]
+pub mod x509;
 
 use caret::caret_int;
 use tor_bytes::{Error as BytesError, Result as BytesResult};
