@@ -13,6 +13,8 @@
 //! a file on disk.
 
 mod ipt;
+#[cfg(feature = "hs-pow-full")]
+mod pow;
 
 use crate::internal_prelude::*;
 
@@ -44,6 +46,10 @@ pub(crate) struct ReplayLog<T> {
 /// A [`ReplayLog`] for [`Introduce2`](tor_cell::relaycell::msg::Introduce2) messages.
 pub(crate) type IptReplayLog = ReplayLog<ipt::IptReplayLogType>;
 
+/// A [`ReplayLog`] for Proof-of-Work [`Nonce`](tor_hscrypto::pow::v1::Nonce)s.
+#[cfg(feature = "hs-pow-full")]
+pub(crate) type PowNonceReplayLog = ReplayLog<pow::PowNonceReplayLogType>;
+
 /// The length of the [`ReplayLogType::MAGIC`] constant.
 ///
 // TODO: If Rust's constant expressions supported generics we wouldn't need this at all.
@@ -135,12 +141,12 @@ impl<T: ReplayLogType> ReplayLog<T> {
     pub(crate) fn new_logged(
         dir: &InstanceRawSubdir,
         name: &T::Name,
-    ) -> Result<Self, CreateIptError> {
+    ) -> Result<Self, OpenReplayLogError> {
         let leaf = T::format_filename(name);
         let path = dir.as_path().join(leaf);
         let lock_guard = dir.raw_lock_guard();
 
-        Self::new_logged_inner(&path, lock_guard).map_err(|error| CreateIptError::OpenReplayLog {
+        Self::new_logged_inner(&path, lock_guard).map_err(|error| OpenReplayLogError {
             file: path,
             error: error.into(),
         })
@@ -298,7 +304,7 @@ impl<T: ReplayLogType> ReplayLog<T> {
 ///
 /// We isolate this code to make it easier to replace.
 mod data {
-    use super::{ReplayError, OUTPUT_LEN};
+    use super::{OUTPUT_LEN, ReplayError};
     use growable_bloom_filter::GrowableBloom;
 
     /// A probabilistic membership filter.
@@ -350,6 +356,17 @@ pub(crate) enum ReplayError {
     Log(Arc<std::io::Error>),
 }
 
+/// Error occured while opening replay log.
+#[derive(thiserror::Error, Clone, Debug)]
+#[error("unable to open replay log: {file:?}")]
+pub struct OpenReplayLogError {
+    /// What filesystem object we tried to do it to
+    pub(crate) file: PathBuf,
+    /// What happened
+    #[source]
+    pub(crate) error: Arc<io::Error>,
+}
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -369,7 +386,7 @@ mod test {
     use super::*;
     use crate::test::mk_state_instance;
     use rand::Rng;
-    use test_temp_dir::{test_temp_dir, TestTempDir, TestTempDirGuard};
+    use test_temp_dir::{TestTempDir, TestTempDirGuard, test_temp_dir};
 
     struct TestReplayLogType;
 
@@ -400,7 +417,7 @@ mod test {
     }
 
     fn rand_msg<R: Rng>(rng: &mut R) -> [u8; OUTPUT_LEN] {
-        rng.gen()
+        rng.random()
     }
 
     /// Basic tests on an ephemeral IptReplayLog.
@@ -649,9 +666,8 @@ mod test {
             }
 
             let demand_efbig = |e| match e {
-                // MSRV:: io::ErrorKind::FileTooLarge is still unstable
-                ReplayError::Log(e) if e.raw_os_error() == Some(libc::EFBIG) => {}
-                other => panic!("expected EFBUG, got {other:?}"),
+                ReplayError::Log(e) if e.kind() == io::ErrorKind::FileTooLarge => {}
+                other => panic!("expected EFBIG, got {other:?}"),
             };
 
             // Generate a distinct message given a phase and a counter
