@@ -13,6 +13,14 @@ pub(crate) mod misc;
 pub mod policy;
 pub mod version;
 
-pub use misc::Nickname;
+pub use misc::{Nickname, NotPresent};
 
-pub use misc::{Fingerprint, Ignored};
+pub use misc::B64;
+pub use misc::{Base64Fingerprint, Fingerprint, Ignored};
+pub use misc::{DigestName, IdentifiedDigest};
+pub use misc::{Iso8601TimeNoSp, Iso8601TimeSp};
+
+use crate::NormalItemArgument;
+
+impl NormalItemArgument for std::net::Ipv4Addr {}
+impl NormalItemArgument for std::net::SocketAddr {}
