@@ -6,7 +6,7 @@
 //!
 //! 1. Is the message contextually appropriate? (e.g., no more than one
 //!    `CONNECTED` message per stream.) This is handled by calling
-//!    [`CmdChecker::check_msg`](crate::client::stream::CmdChecker::check_msg).
+//!    [`CmdChecker::check_msg`](crate::stream::cmdcheck::CmdChecker::check_msg).
 //! 2. Does the message comply with flow-control rules? (e.g., no more SENDMEs
 //!    than we've sent data for.) This is handled within the reactor by the
 //!    `StreamFlowCtrl`. For half-closed streams which don't send stream
@@ -17,25 +17,27 @@
 //!    For half-closed streams, the reactor handles it by calling
 //!    `consume_checked_msg()`.
 
-pub(super) mod circuit;
+pub(crate) mod circuit;
 mod conflux;
 mod control;
 pub(super) mod syncview;
 
 use crate::circuit::UniqId;
-use crate::client::circuit::celltypes::ClientCircChanMsg;
+use crate::circuit::celltypes::ClientCircChanMsg;
 use crate::client::circuit::padding::{PaddingController, PaddingEvent, PaddingEventStream};
 use crate::client::circuit::{CircuitRxReceiver, TimeoutEstimator};
-use crate::client::stream::AnyCmdChecker;
-use crate::client::stream::flow_ctrl::state::StreamRateLimit;
-use crate::client::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
-use crate::client::stream::queue::StreamQueueReceiver;
 #[cfg(feature = "hs-service")]
 use crate::client::stream::{IncomingStreamRequest, IncomingStreamRequestFilter};
-use crate::client::{HopLocation, TargetHop, streammap};
+use crate::client::{HopLocation, TargetHop};
 use crate::crypto::cell::HopNum;
 use crate::crypto::handshake::ntor_v3::NtorV3PublicKey;
 use crate::memquota::{CircuitAccount, StreamAccount};
+use crate::stream::StreamMpscSender;
+use crate::stream::cmdcheck::AnyCmdChecker;
+use crate::stream::flow_ctrl::state::StreamRateLimit;
+use crate::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
+use crate::stream::queue::StreamQueueReceiver;
+use crate::streammap;
 use crate::tunnel::{TunnelId, TunnelScopedCircId};
 use crate::util::err::ReactorError;
 use crate::util::notify::NotifyReceiver;
@@ -65,7 +67,6 @@ use std::sync::Arc;
 use std::time::Duration;
 
 use crate::channel::Channel;
-use crate::client::circuit::StreamMpscSender;
 use crate::conflux::msghandler::RemoveLegReason;
 use crate::crypto::handshake::ntor::{NtorClient, NtorPublicKey};
 use circuit::CircuitCmd;
@@ -524,12 +525,6 @@ macro_rules! unwrap_or_shutdown {
     }};
 }
 
-// Note: this is used in the relay reactor too
-//
-// TODO(relay): refactor the common parts of the two reactors
-// into a shared module
-pub(crate) use unwrap_or_shutdown;
-
 /// Object to handle incoming cells and background tasks on a circuit
 ///
 /// This type is returned when you finish a circuit; you need to spawn a
