@@ -7,22 +7,16 @@ pub(super) mod extender;
 
 use crate::channel::Channel;
 use crate::circuit::UniqId;
-use crate::client::circuit::celltypes::{ClientCircChanMsg, CreateResponse};
-#[cfg(feature = "counter-galois-onion")]
-use crate::client::circuit::handshake::RelayCryptLayerProtocol;
+use crate::circuit::celltypes::{ClientCircChanMsg, CreateResponse};
+use crate::circuit::circhop::HopSettings;
 use crate::client::circuit::handshake::{BoxedClientLayer, HandshakeRole};
 use crate::client::circuit::padding::{
     self, PaddingController, PaddingEventStream, QueuedCellPaddingInfo,
 };
-use crate::client::circuit::{CircuitRxReceiver, MutableState, StreamMpscReceiver};
-use crate::client::circuit::{HopSettings, TimeoutEstimator, path};
+use crate::client::circuit::{CircuitRxReceiver, MutableState};
+use crate::client::circuit::{TimeoutEstimator, path};
 use crate::client::reactor::MetaCellDisposition;
 use crate::client::reactor::circuit::cell_sender::CircuitCellSender;
-use crate::client::stream::flow_ctrl::state::StreamRateLimit;
-use crate::client::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
-use crate::client::stream::queue::{StreamQueueSender, stream_queue};
-use crate::client::stream::{AnyCmdChecker, StreamStatus};
-use crate::client::streammap;
 use crate::congestion::CongestionSignals;
 use crate::congestion::sendme;
 use crate::crypto::binding::CircuitBinding;
@@ -35,6 +29,12 @@ use crate::crypto::handshake::ntor::{NtorClient, NtorPublicKey};
 use crate::crypto::handshake::ntor_v3::{NtorV3Client, NtorV3PublicKey};
 use crate::crypto::handshake::{ClientHandshake, KeyGenerator};
 use crate::memquota::{CircuitAccount, SpecificAccount as _, StreamAccount};
+use crate::stream::StreamMpscReceiver;
+use crate::stream::cmdcheck::{AnyCmdChecker, StreamStatus};
+use crate::stream::flow_ctrl::state::StreamRateLimit;
+use crate::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
+use crate::stream::queue::{StreamQueueSender, stream_queue};
+use crate::streammap;
 use crate::tunnel::TunnelScopedCircId;
 use crate::util::err::ReactorError;
 use crate::util::notify::NotifySender;
@@ -44,7 +44,6 @@ use tor_async_utils::{SinkTrySend as _, SinkTrySendError as _};
 use tor_cell::chancell::msg::{AnyChanMsg, HandshakeType, Relay};
 use tor_cell::chancell::{AnyChanCell, ChanCmd, CircId};
 use tor_cell::chancell::{BoxedCellBody, ChanMsg};
-use tor_cell::relaycell::extend::{CcRequest, CircRequestExt};
 use tor_cell::relaycell::msg::{AnyRelayMsg, End, Sendme, SendmeTag, Truncated};
 use tor_cell::relaycell::{
     AnyRelayMsgOuter, RelayCellDecoderResult, RelayCellFormat, RelayCmd, StreamId, UnparsedRelayMsg,
@@ -78,7 +77,7 @@ use extender::HandshakeAuxDataHandler;
 
 #[cfg(feature = "hs-service")]
 use {
-    crate::client::stream::{DataCmdChecker, IncomingStreamRequest},
+    crate::client::stream::{InboundDataCmdChecker, IncomingStreamRequest},
     tor_cell::relaycell::msg::Begin,
 };
 
@@ -91,7 +90,7 @@ use {
 pub(super) use circhop::{CircHop, CircHopList};
 
 /// Initial value for outbound flow-control window on streams.
-pub(super) const SEND_WINDOW_INIT: u16 = 500;
+pub(crate) const SEND_WINDOW_INIT: u16 = 500;
 /// Initial value for inbound flow-control window on streams.
 pub(crate) const RECV_WINDOW_INIT: u16 = 500;
 /// Size of the buffer used between the reactor and a `StreamReader`.
@@ -368,7 +367,7 @@ impl Circuit {
 
         let settings = HopSettings::from_params_and_caps(
             // This is for testing only, so we'll assume full negotiation took place.
-            crate::client::circuit::HopNegotiationType::Full,
+            crate::circuit::circhop::HopNegotiationType::Full,
             params,
             &[named::FLOWCTRL_CC].into_iter().collect::<Protocols>(),
         )
@@ -943,7 +942,7 @@ impl Circuit {
         let mut drain_rate_request_tx = NotifySender::new_typed();
         let drain_rate_request_rx = drain_rate_request_tx.subscribe();
 
-        let cmd_checker = DataCmdChecker::new_connected();
+        let cmd_checker = InboundDataCmdChecker::new_connected();
         hop.add_ent_with_id(
             sender,
             msg_rx,
@@ -1177,7 +1176,7 @@ impl Circuit {
         self.channel.check_match(&target)?;
 
         // Set the client extensions.
-        let client_extensions = circ_extensions_from_settings(&settings)?;
+        let client_extensions = settings.circuit_request_extensions()?;
         let wrap = Create2Wrap {
             handshake_type: HandshakeType::NTOR_V3,
         };
@@ -1754,60 +1753,3 @@ impl Drop for Circuit {
         let _ = self.channel.close_circuit(self.channel_id);
     }
 }
-
-/// Return the client circuit-creation extensions that we should use in order to negotiate
-/// a given set of circuit hop parameters.
-#[allow(clippy::unnecessary_wraps)]
-pub(super) fn circ_extensions_from_settings(params: &HopSettings) -> Result<Vec<CircRequestExt>> {
-    // allow 'unused_mut' because of the combinations of `cfg` conditions below
-    #[allow(unused_mut)]
-    let mut client_extensions = Vec::new();
-
-    #[allow(unused, unused_mut)]
-    let mut cc_extension_set = false;
-
-    if params.ccontrol.is_enabled() {
-        cfg_if::cfg_if! {
-            if #[cfg(feature = "flowctl-cc")] {
-                client_extensions.push(CircRequestExt::CcRequest(CcRequest::default()));
-                cc_extension_set = true;
-            } else {
-                return Err(
-                    tor_error::internal!(
-                        "Congestion control is enabled on this circuit, but 'flowctl-cc' feature is not enabled"
-                    )
-                    .into()
-                );
-            }
-        }
-    }
-
-    // See whether we need to send a list of required protocol capabilities.
-    // These aren't "negotiated" per se; they're simply demanded.
-    // The relay will refuse the circuit if it doesn't support all of them,
-    // and if any of them isn't supported in the SubprotocolRequest extension.
-    //
-    // (In other words, don't add capabilities here just because you want the
-    // relay to have them! They must be explicitly listed as supported for use
-    // with this extension. For the current list, see
-    // https://spec.torproject.org/tor-spec/create-created-cells.html#subproto-request)
-    //
-    #[allow(unused_mut)]
-    let mut required_protocol_capabilities: Vec<tor_protover::NamedSubver> = Vec::new();
-
-    #[cfg(feature = "counter-galois-onion")]
-    if matches!(params.relay_crypt_protocol(), RelayCryptLayerProtocol::Cgo) {
-        if !cc_extension_set {
-            return Err(tor_error::internal!("Tried to negotiate CGO without CC.").into());
-        }
-        required_protocol_capabilities.push(tor_protover::named::RELAY_CRYPT_CGO);
-    }
-
-    if !required_protocol_capabilities.is_empty() {
-        client_extensions.push(CircRequestExt::SubprotocolRequest(
-            required_protocol_capabilities.into_iter().collect(),
-        ));
-    }
-
-    Ok(client_extensions)
-}
