@@ -2,21 +2,18 @@
 
 use super::CircuitCmd;
 use super::{CloseStreamBehavior, SEND_WINDOW_INIT, SendRelayCell};
-use crate::client::circuit::{HopSettings, StreamMpscReceiver};
+use crate::circuit::circhop::HopSettings;
 use crate::client::reactor::circuit::path::PathEntry;
-use crate::client::stream::flow_ctrl::params::FlowCtrlParameters;
-use crate::client::stream::flow_ctrl::state::{
-    StreamEndpointType, StreamFlowCtrl, StreamRateLimit,
-};
-use crate::client::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
-use crate::client::stream::queue::StreamQueueSender;
-use crate::client::stream::{AnyCmdChecker, StreamStatus};
-use crate::client::streammap::{
-    self, EndSentStreamEnt, OpenStreamEnt, ShouldSendEnd, StreamEntMut,
-};
 use crate::congestion::CongestionControl;
 use crate::congestion::sendme;
 use crate::crypto::cell::HopNum;
+use crate::stream::StreamMpscReceiver;
+use crate::stream::cmdcheck::{AnyCmdChecker, StreamStatus};
+use crate::stream::flow_ctrl::params::FlowCtrlParameters;
+use crate::stream::flow_ctrl::state::{StreamFlowCtrl, StreamRateLimit};
+use crate::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
+use crate::stream::queue::StreamQueueSender;
+use crate::streammap::{self, EndSentStreamEnt, OpenStreamEnt, ShouldSendEnd, StreamEntMut};
 use crate::tunnel::TunnelScopedCircId;
 use crate::util::notify::NotifySender;
 use crate::{Error, Result};
@@ -25,6 +22,7 @@ use futures::Stream;
 use futures::stream::FuturesUnordered;
 use postage::watch;
 use safelog::sensitive as sv;
+use smallvec::SmallVec;
 use tor_cell::chancell::BoxedCellBody;
 use tor_cell::relaycell::flow_ctrl::{Xoff, Xon, XonKbpsEwma};
 use tor_cell::relaycell::msg::AnyRelayMsg;
@@ -46,11 +44,16 @@ use std::time::Instant;
 #[cfg(test)]
 use tor_cell::relaycell::msg::SendmeTag;
 
+/// The "usual" number of hops in a [`CircHopList`].
+///
+/// This saves us a heap allocation when the number of hops is less than or equal to this value.
+const NUM_HOPS: usize = 3;
+
 /// Represents the reactor's view of a circuit's hop.
 #[derive(Default)]
 pub(crate) struct CircHopList {
     /// The list of hops.
-    hops: Vec<CircHop>,
+    hops: SmallVec<[CircHop; NUM_HOPS]>,
 }
 
 impl CircHopList {
@@ -601,20 +604,19 @@ impl CircHop {
             cfg_if::cfg_if! {
                 if #[cfg(feature = "flowctl-cc")] {
                     // TODO: Currently arti only supports clients, and we don't support connecting
-                    // to onion services while using congestion control, so we hardcode these. In
-                    // the future we will need to somehow tell the `CircHop` these things so that we
-                    // can set them correctly.
-                    let our_endpoint = StreamEndpointType::Client;
-                    let peer_endpoint = StreamEndpointType::Exit;
+                    // to onion services while using congestion control, so we hardcode this. In the
+                    // future we will need to somehow tell the `CircHop` this so that we can set it
+                    // correctly, since we don't want to enable this at exits.
+                    let use_sidechannel_mitigations = true;
 
                     Ok(StreamFlowCtrl::new_xon_xoff(
                         params,
-                        our_endpoint,
-                        peer_endpoint,
+                        use_sidechannel_mitigations,
                         rate_limit_updater,
                         drain_rate_requester,
                     ))
                 } else {
+                    drop(params);
                     drop(rate_limit_updater);
                     drop(drain_rate_requester);
                     Err(internal!(
