@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -41,6 +41,7 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 // TODO #1645 (either remove this, or decide to have it everywhere)
@@ -48,7 +49,7 @@
 
 #[cfg(all(
     any(feature = "native-tls", feature = "rustls"),
-    any(feature = "async-std", feature = "tokio")
+    any(feature = "async-std", feature = "tokio", feature = "smol")
 ))]
 pub(crate) mod impls;
 pub mod task;
@@ -64,12 +65,12 @@ mod traits;
 pub mod unimpl;
 pub mod unix;
 
-#[cfg(any(feature = "async-std", feature = "tokio"))]
+#[cfg(any(feature = "async-std", feature = "tokio", feature = "smol"))]
 use std::io;
 pub use traits::{
-    BlockOn, CertifiedConn, CoarseTimeProvider, NetStreamListener, NetStreamProvider,
-    NoOpStreamOpsHandle, Runtime, SleepProvider, SpawnBlocking, StreamOps, TlsProvider,
-    UdpProvider, UdpSocket, UnsupportedStreamOp,
+    Blocking, CertifiedConn, CoarseTimeProvider, NetStreamListener, NetStreamProvider,
+    NoOpStreamOpsHandle, Runtime, SleepProvider, StreamOps, TlsProvider, ToplevelBlockOn,
+    ToplevelRuntime, UdpProvider, UdpSocket, UnsupportedStreamOp,
 };
 
 pub use coarse_time::{CoarseDuration, CoarseInstant, RealCoarseTimeProvider};
@@ -81,9 +82,15 @@ pub use timer::{SleepProviderExt, Timeout, TimeoutError};
 pub mod tls {
     pub use crate::traits::{CertifiedConn, TlsConnector};
 
-    #[cfg(all(feature = "native-tls", any(feature = "tokio", feature = "async-std")))]
+    #[cfg(all(
+        feature = "native-tls",
+        any(feature = "tokio", feature = "async-std", feature = "smol")
+    ))]
     pub use crate::impls::native_tls::NativeTlsProvider;
-    #[cfg(all(feature = "rustls", any(feature = "tokio", feature = "async-std")))]
+    #[cfg(all(
+        feature = "rustls",
+        any(feature = "tokio", feature = "async-std", feature = "smol")
+    ))]
     pub use crate::impls::rustls::RustlsProvider;
 }
 
@@ -93,6 +100,9 @@ pub mod tokio;
 #[cfg(all(any(feature = "native-tls", feature = "rustls"), feature = "async-std"))]
 pub mod async_std;
 
+#[cfg(all(any(feature = "native-tls", feature = "rustls"), feature = "smol"))]
+pub mod smol;
+
 pub use compound::{CompoundRuntime, RuntimeSubstExt};
 
 #[cfg(all(
@@ -111,6 +121,10 @@ use tokio as preferred_backend_mod;
 /// performance.
 /// If `native_tls` and `rustls` are both available, we prefer `native_tls` since
 /// it has been used in Arti for longer.
+///
+/// The process [**may not fork**](crate#do-not-fork)
+/// (except, very carefully, before exec)
+/// after creating this or any other `Runtime`.
 #[cfg(all(
     any(feature = "native-tls", feature = "rustls"),
     any(feature = "async-std", feature = "tokio")
@@ -158,6 +172,9 @@ impl PreferredRuntime {
     /// If the `tor-rtcompat` crate was compiled with `tokio` support,
     /// this function will never return a runtime based on `async_std`.
     ///
+    /// The process [**may not fork**](crate#do-not-fork)
+    /// (except, very carefully, before exec)
+    /// after creating this or any other `Runtime`.
     //
     // ## Note to Arti developers
     //
@@ -183,6 +200,10 @@ impl PreferredRuntime {
     ///
     /// If you need more fine-grained control over a runtime, you can create it
     /// using an appropriate builder type or function.
+    ///
+    /// The process [**may not fork**](crate#do-not-fork)
+    /// (except, very carefully, before exec)
+    /// after creating this or any other `Runtime`.
     //
     // ## Note to Arti developers
     //
@@ -289,6 +310,16 @@ pub mod cond {
         #[doc(hidden)]
         macro if_async_std_rustls_present = ("async-std", "rustls")
     }
+    declare_conditional_macro! {
+        /// Expand a token tree if the SmolNativeTlsRuntime is available.
+        #[doc(hidden)]
+        macro if_smol_native_tls_present = ("smol", "native-tls")
+    }
+    declare_conditional_macro! {
+        /// Expand a token tree if the SmolRustlsRuntime is available.
+        #[doc(hidden)]
+        macro if_smol_rustls_present = ("smol", "rustls")
+    }
 }
 
 /// Run a test closure, passing as argument every supported runtime.
@@ -309,7 +340,7 @@ pub mod cond {
 #[macro_export]
 #[cfg(all(
     any(feature = "native-tls", feature = "rustls"),
-    any(feature = "tokio", feature = "async-std"),
+    any(feature = "tokio", feature = "async-std", feature = "smol"),
 ))]
 macro_rules! test_with_all_runtimes {
     ( $fn:expr ) => {{
@@ -331,6 +362,12 @@ macro_rules! test_with_all_runtimes {
         if_async_std_rustls_present! {{
             $crate::async_std::AsyncStdRustlsRuntime::run_test($fn).check_ok();
         }}
+        if_smol_native_tls_present! {{
+            $crate::smol::SmolNativeTlsRuntime::run_test($fn).check_ok();
+        }}
+        if_smol_rustls_present! {{
+            $crate::smol::SmolRustlsRuntime::run_test($fn).check_ok();
+        }}
     }};
 }
 
@@ -351,21 +388,19 @@ macro_rules! test_with_all_runtimes {
     any(feature = "tokio", feature = "async-std"),
 ))]
 macro_rules! test_with_one_runtime {
-    ( $fn:expr ) => {{
-        $crate::PreferredRuntime::run_test($fn)
-    }};
+    ( $fn:expr ) => {{ $crate::PreferredRuntime::run_test($fn) }};
 }
 
 #[cfg(all(
     test,
     any(feature = "native-tls", feature = "rustls"),
-    any(feature = "async-std", feature = "tokio"),
-    not(miri), // Many of these tests use real sockets or SystemTime
+    any(feature = "async-std", feature = "tokio", feature = "smol"),
+    not(miri), // Many of these tests use real sockets or SystemTime.
 ))]
 mod test {
     #![allow(clippy::unwrap_used, clippy::unnecessary_wraps)]
-    use crate::Runtime;
     use crate::SleepProviderExt;
+    use crate::ToplevelRuntime;
 
     use crate::traits::*;
 
@@ -379,7 +414,7 @@ mod test {
 
     // Test "sleep" with a tiny delay, and make sure that at least that
     // much delay happens.
-    fn small_delay<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn small_delay<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let rt = runtime.clone();
         runtime.block_on(async {
             let i1 = Instant::now();
@@ -392,7 +427,7 @@ mod test {
     }
 
     // Try a timeout operation that will succeed.
-    fn small_timeout_ok<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn small_timeout_ok<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let rt = runtime.clone();
         runtime.block_on(async {
             let one_day = Duration::from_secs(86400);
@@ -403,7 +438,7 @@ mod test {
     }
 
     // Try a timeout operation that will time out.
-    fn small_timeout_expire<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn small_timeout_expire<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         use futures::future::pending;
 
         let rt = runtime.clone();
@@ -422,7 +457,7 @@ mod test {
     //
     // NOTE: This test will fail if the clock jumps a lot while it's
     // running.  We should use simulated time instead.
-    fn tiny_wallclock<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn tiny_wallclock<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let rt = runtime.clone();
         runtime.block_on(async {
             let i1 = Instant::now();
@@ -443,7 +478,7 @@ mod test {
     // Try connecting to ourself and sending a little data.
     //
     // NOTE: requires Ipv4 localhost.
-    fn self_connect_tcp<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn self_connect_tcp<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let localhost = SocketAddrV4::new(Ipv4Addr::LOCALHOST, 0);
         let rt1 = runtime.clone();
 
@@ -476,7 +511,7 @@ mod test {
     // Try connecting to ourself and sending a little data.
     //
     // NOTE: requires Ipv4 localhost.
-    fn self_connect_udp<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn self_connect_udp<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let localhost = SocketAddrV4::new(Ipv4Addr::LOCALHOST, 0);
         let rt1 = runtime.clone();
 
@@ -511,7 +546,7 @@ mod test {
     //
     // We launch a few connections and make sure that we can read data on
     // them.
-    fn listener_stream<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn listener_stream<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let localhost = SocketAddrV4::new(Ipv4Addr::LOCALHOST, 0);
         let rt1 = runtime.clone();
 
@@ -560,7 +595,7 @@ mod test {
     //
     // Note that since we don't have async tls server support yet, I'm just
     // going to use a thread.
-    fn simple_tls<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn simple_tls<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         /*
          A simple expired self-signed rsa-2048 certificate.
 
@@ -642,6 +677,10 @@ mod test {
             mod async_std_runtime_tests {
                 tests_with_runtime! { &crate::async_std::PreferredRuntime::create()? => $($id),* }
             }
+            #[cfg(feature="smol")]
+            mod smol_runtime_tests {
+                tests_with_runtime! { &crate::smol::PreferredRuntime::create()? => $($id),* }
+            }
             mod default_runtime_tests {
                 tests_with_runtime! { &crate::PreferredRuntime::create()? => $($id),* }
             }
@@ -659,6 +698,10 @@ mod test {
             mod async_std_native_tls_tests {
                 tests_with_runtime! { &crate::async_std::AsyncStdNativeTlsRuntime::create()? => $($id),* }
             }
+            #[cfg(all(feature="smol", feature = "native-tls"))]
+            mod smol_native_tls_tests {
+                tests_with_runtime! { &crate::smol::SmolNativeTlsRuntime::create()? => $($id),* }
+            }
             #[cfg(all(feature="tokio", feature="rustls"))]
             mod tokio_rustls_tests {
                 tests_with_runtime! {  &crate::tokio::TokioRustlsRuntime::create()? => $($id),* }
@@ -667,6 +710,10 @@ mod test {
             mod async_std_rustls_tests {
                 tests_with_runtime! {  &crate::async_std::AsyncStdRustlsRuntime::create()? => $($id),* }
             }
+            #[cfg(all(feature="smol", feature="rustls"))]
+            mod smol_rustls_tests {
+                tests_with_runtime! {  &crate::smol::SmolRustlsRuntime::create()? => $($id),* }
+            }
             mod default_runtime_tls_tests {
                 tests_with_runtime! { &crate::PreferredRuntime::create()? => $($id),* }
             }
