@@ -70,11 +70,6 @@ use crate::time_core::MockTimeCore;
 ///    coordination with the executor is required.
 ///    This coordination is provided by the integrated `MockRuntime`;
 ///    `SimpleMockTimeProvider` is of limited usefulness by itself.
-//
-// TODO: at some point we should add #[deprecated] to this type
-// and to the block_advance etc. methods in SleepProvider.
-// But right now that would involve rewriting a whole bunch of tests,
-// or generous sprinklings of #[allow].
 ///
 /// ### Examples
 ///
@@ -152,6 +147,10 @@ use crate::time_core::MockTimeCore;
 /// });
 /// ```
 #[derive(Clone)]
+// When we're used by external crates, we're always cfg(not(test)), so we seem deprecated
+// from outside this crate.  *Within* this crate, this cfg_attr means that if we use things
+// that are deprecated for other reasons, we will notice.
+#[cfg_attr(not(test), deprecated(since = "0.29.0"))]
 pub struct MockSleepProvider {
     /// The shared backend for this MockSleepProvider and its futures.
     state: Arc<Mutex<SleepSchedule>>,
@@ -348,8 +347,7 @@ impl MockSleepProvider {
                 // The next timeout is too far in the future.
                 trace!(
                     "should_advance = false; allow_one set but only up to {:?}, next is {:?}",
-                    state.allowed_advance,
-                    next_timeout
+                    state.allowed_advance, next_timeout
                 );
                 return false;
             }
@@ -420,8 +418,7 @@ impl SleepSchedule {
         self.sleepers_polled += 1;
         trace!(
             "sleeper polled, {}/{}",
-            self.sleepers_polled,
-            self.sleepers_made
+            self.sleepers_polled, self.sleepers_made
         );
         self.maybe_advance();
     }
@@ -436,9 +433,7 @@ impl SleepProvider for MockSleepProvider {
         provider.sleepers_made += 1;
         trace!(
             "sleeper made for {:?}, {}/{}",
-            duration,
-            provider.sleepers_polled,
-            provider.sleepers_made
+            duration, provider.sleepers_polled, provider.sleepers_made
         );
 
         Sleeping {
